;NSIS Modern User Interface version 2.03b
;Massiv simulation install script
;Written by Martin Havlista

!define MUI_PRODUCT "Massiv Simulation" ;Define your own software name here
!define MUI_VERSION "1.01" ;Define your own software version here

!include "MUI.nsh"
!include "Sections.nsh"

!define TEMP $R0

!include "msvc.nsi"

;--------------------------------
;Configuration

  ;General
  OutFile "simulation_install.exe"

  ;Folder selection page
  InstallDir "$PROGRAMFILES\${MUI_PRODUCT}"

  ;Remember install folder
;  InstallDirRegKey HKCU "Software\massiv\${MUI_PRODUCT}" ""

;--------------------------------
;Modern UI Configuration

  !define MUI_CUSTOMPAGECOMMANDS

  !define MUI_WELCOMEPAGE
;  !define MUI_LICENSEPAGE
  !define MUI_COMPONENTSPAGE
  !define MUI_DIRECTORYPAGE
  !define MUI_STARTMENUPAGE
    !define MUI_STARTMENUPAGE_REGISTRY_ROOT "HKLM"
    !define MUI_STARTMENUPAGE_REGISTRY_KEY "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}"
    !define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu Folder"
    !define MUI_STARTMENUPAGE_VARIABLE $R9

  !define MUI_FINISHPAGE
    !define MUI_FINISHPAGE_RUN "$INSTDIR\start_all.bat"

  !define MUI_ABORTWARNING

  !define MUI_UNINSTALLER
  !define MUI_UNCONFIRMPAGE

;--------------------------------
;Pages

  !insertmacro MUI_PAGECOMMAND_WELCOME
;  !insertmacro MUI_PAGECOMMAND_LICENSE
  Page custom SetPage1
  Page custom SetPage2
  Page custom SetPage3
  Page custom SetPage4
  Page custom SetPage5
  !insertmacro MUI_PAGECOMMAND_COMPONENTS
  !insertmacro MUI_PAGECOMMAND_DIRECTORY
  !insertmacro MUI_PAGECOMMAND_STARTMENU
  !insertmacro MUI_PAGECOMMAND_INSTFILES
  !insertmacro MUI_PAGECOMMAND_FINISH
;--------------------------------
;Languages

  !insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Language Strings

  ;Description
  LangString DESC_SecServiceUI ${LANG_ENGLISH} "Install Master data service."
  LangString DESC_SecServerUI1 ${LANG_ENGLISH} "Install Server 1."
  LangString DESC_SecServerUI2 ${LANG_ENGLISH} "Install Server 2."
  LangString DESC_SecServerUI3 ${LANG_ENGLISH} "Install Server 3."
  LangString DESC_SecServerUI4 ${LANG_ENGLISH} "Install Server 4."
  LangString DESC_SecServerUI5 ${LANG_ENGLISH} "Install Server 5."
  LangString DESC_SecServerUI6 ${LANG_ENGLISH} "Install Server 6."
  LangString DESC_SecServerUI7 ${LANG_ENGLISH} "Install Server 7."
  LangString DESC_SecServerUI8 ${LANG_ENGLISH} "Install Server 8."

  ;Header
  LangString TEXT_IO_TITLE ${LANG_ENGLISH} "Massiv Simulation Setup"
  LangString TEXT_IO_SUBTITLE ${LANG_ENGLISH} " "

;--------------------------------
;Data

;  LicenseData "${NSISDIR}\Contrib\Modern UI\License.txt"

;--------------------------------
;Reserve Files

  ;Things that need to be extracted on first (keep these lines before any File command!)
  ;Only useful for BZIP2 compression
  !insertmacro MUI_RESERVEFILE_WELCOMEFINISHPAGE

  ReserveFile "init.ini"
  ReserveFile "generate_setup.ini"
  ReserveFile "generate_process.ini"
  ReserveFile "install.ini"
  ReserveFile "setup.ini"
  ReserveFile "password.ini"

;--------------------------------
;Installer Sections

SubSection /e "Service nodes" one

Section "Master data service" SecServiceUI

  ; common files
  SetOutPath "$INSTDIR"
  File "common\massiv.logger.conf"
  File "common\massiv.replication.conf"
  File "common\start_all.bat"

  ; dll files
  SetOutPath "$INSTDIR\master_data_service"
  Call copy_msvc_dll_files

  ; Service files
  SetOutPath "$INSTDIR\master_data_service"
  File "service\demo_service.exe"
  File "service\massiv.settings.conf"
  File "service\massiv.conf"
  File "service\massiv.paths.conf"
  File "service\source_data.zip"

  ZipDLL::extractall "$INSTDIR\master_data_service\source_data.zip" "$INSTDIR\master_data_service"
  Pop ${TEMP}
  StrCmp ${TEMP} "success" done
    MessageBox MB_OK "Cannot extract files from archive source_data.zip. Error message: ${TEMP}"
    Abort
  done:

  Delete "$INSTDIR\master_data_service\source_data.zip"

  ClearErrors
  CopyFiles /SILENT "$0\massiv.server_nodes.conf" "$INSTDIR\massiv.server_nodes.conf"
  IfErrors error

  ClearErrors
  CopyFiles /SILENT "$0\master_data_service\source_data\config_nodes_public" "$INSTDIR\master_data_service\source_data\config_nodes_public"
  CopyFiles /SILENT "$0\master_data_service\source_data\config_nodes_private" "$INSTDIR\master_data_service\source_data\config_nodes_private"
  IfErrors error

  ClearErrors
  CreateDirectory "$INSTDIR\master_data_service\login"
  IfErrors error

  ClearErrors
  CopyFiles /SILENT "$0\master_data_service\login\default" "$INSTDIR\master_data_service\login\default"
  IfErrors error

  Goto next
  error:
  MessageBox MB_OK "Error while processing generated config files. $0"
  Abort

  next:
  
  ; write registry
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "DisplayName" "${MUI_PRODUCT}"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "UninstallString" '"$INSTDIR\Uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "Service1" "1"
  ; write generated directory
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "GeneratedDirectory" "$0"

  ;Create uninstaller
  SetOutPath "$INSTDIR"
  WriteUninstaller "$INSTDIR\Uninstall.exe"

  !insertmacro MUI_STARTMENU_WRITE_BEGIN
  ; FIXME - dat na vyber, jestli instalovat pro jednoho uzivatele nebo pro vsechny uzivatele
  SetShellVarContext all
  CreateDirectory "$SMPROGRAMS\${MUI_STARTMENUPAGE_VARIABLE}"
  ;IfFileExists "$SMPROGRAMS\${MUI_STARTMENUPAGE_VARIABLE}" createIcons
    ;SetShellVarContext current
    ;CreateDirectory "$SMPROGRAMS\${MUI_STARTMENUPAGE_VARIABLE}"

  ;createIcons:
    SetOutPath "$INSTDIR\master_data_service"
    CreateShortcut "$SMPROGRAMS\${MUI_STARTMENUPAGE_VARIABLE}\Master data service.lnk" "$INSTDIR\master_data_service\demo_service.exe" ""
    CreateShortCut "$SMPROGRAMS\${MUI_STARTMENUPAGE_VARIABLE}\Uninstall.lnk" "$INSTDIR\Uninstall.exe"

    SetOutPath "$INSTDIR"
    CreateShortCut "$SMPROGRAMS\${MUI_STARTMENUPAGE_VARIABLE}\Start all.lnk" "$INSTDIR\start_all.bat"
  !insertmacro MUI_STARTMENU_WRITE_END

SectionEnd

SubSectionEnd

Function server_section

  Pop $R1 ; server number

  ; common files
  SetOutPath "$INSTDIR"
  File "common\massiv.logger.conf"
  File "common\massiv.replication.conf"
  File "common\start_all.bat"

  ; dll files
  SetOutPath "$INSTDIR\server$R1"
  Call copy_msvc_dll_files

  ; server files
  SetOutPath "$INSTDIR\server$R1"
  File "server\demo_server.exe"
  File "server\massiv.settings.conf"
  File "server\massiv.conf"
  File "server\massiv.paths.conf"

  ;archive files
  SetOutPath "$INSTDIR\server$R1"
  File "server\archive\archive.zip"

  ZipDLL::extractfile "$INSTDIR\server$R1\archive.zip" "$INSTDIR" "server$R1\archives\regular\massiv_0000000001.archive"
  Pop ${TEMP}
  StrCmp ${TEMP} "success" done
    MessageBox MB_OK "Cannot extract files from archive archive.zip. Error message: ${TEMP}"
    Abort
  done:

  Delete "$INSTDIR\server$R1\archive.zip"

  SetOutPath "$INSTDIR\server$R1\archives\regular"
  File "server\archive\massiv_0000000001.archive.__ack__"

  ;generated files
  ClearErrors
  CopyFiles /SILENT "$0\massiv.server_nodes.conf" "$INSTDIR\massiv.server_nodes.conf"
  IfErrors error

  ClearErrors
  CreateDirectory "$INSTDIR\server$R1\login"
  IfErrors error

  ClearErrors
  CopyFiles /SILENT "$0\server$R1\login\default" "$INSTDIR\server$R1\login\default"
  IfErrors error

  Goto next
  error:
  MessageBox MB_OK "Error while processing generated config files."
  Abort

  next:

  ; write registry
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "DisplayName" "${MUI_PRODUCT}"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "UninstallString" '"$INSTDIR\Uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "Server$R1" "1"

  ;Create uninstaller
  SetOutPath "$INSTDIR"
  WriteUninstaller "$INSTDIR\Uninstall.exe"

  !insertmacro MUI_STARTMENU_WRITE_BEGIN
  ; FIXME - dat na vyber, jestli instalovat pro jednoho uzivatele nebo pro vsechny uzivatele
  SetShellVarContext all
  CreateDirectory "$SMPROGRAMS\${MUI_STARTMENUPAGE_VARIABLE}"
  ;IfFileExists "$SMPROGRAMS\${MUI_STARTMENUPAGE_VARIABLE}" createIcons
    ;SetShellVarContext current
    ;CreateDirectory "$SMPROGRAMS\${MUI_STARTMENUPAGE_VARIABLE}"

  ;createIcons:
    SetOutPath "$INSTDIR\server$R1"
    CreateShortcut "$SMPROGRAMS\${MUI_STARTMENUPAGE_VARIABLE}\Server $R1.lnk" "$INSTDIR\server$R1\demo_server.exe" ""
    CreateShortCut "$SMPROGRAMS\${MUI_STARTMENUPAGE_VARIABLE}\Uninstall.lnk" "$INSTDIR\Uninstall.exe"

    SetOutPath "$INSTDIR"
    CreateShortCut "$SMPROGRAMS\${MUI_STARTMENUPAGE_VARIABLE}\Start all.lnk" "$INSTDIR\start_all.bat"
  !insertmacro MUI_STARTMENU_WRITE_END

FunctionEnd

SubSection /e "Server nodes" two

Section "Server 1" SecServerUI1
  Push "1"
  call server_section
SectionEnd
Section "Server 2" SecServerUI2
  Push "2"
  call server_section
SectionEnd
Section "Server 3" SecServerUI3
  Push "3"
  call server_section
SectionEnd
Section "Server 4" SecServerUI4
  Push "4"
  call server_section
SectionEnd
Section "Server 5" SecServerUI5
  Push "5"
  call server_section
SectionEnd
Section "Server 6" SecServerUI6
  Push "6"
  call server_section
SectionEnd
Section "Server 7" SecServerUI7
  Push "7"
  call server_section
SectionEnd
Section "Server 8" SecServerUI8
  Push "8"
  call server_section
SectionEnd

SubSectionEnd

;Display the Finish header
;Insert this macro after the sections if you are not using a finish page
!insertmacro MUI_SECTIONS_FINISHHEADER

;--------------------------------
;Descriptions

!insertmacro MUI_FUNCTIONS_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${SecServiceUI} $(DESC_SecServiceUI)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecServerUI1} $(DESC_SecServerUI1)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecServerUI2} $(DESC_SecServerUI2)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecServerUI3} $(DESC_SecServerUI3)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecServerUI4} $(DESC_SecServerUI4)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecServerUI5} $(DESC_SecServerUI5)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecServerUI6} $(DESC_SecServerUI6)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecServerUI7} $(DESC_SecServerUI7)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecServerUI8} $(DESC_SecServerUI8)
!insertmacro MUI_FUNCTIONS_DESCRIPTION_END

;--------------------------------
;Installer Functions

Function SetPage1
  !insertmacro MUI_HEADER_TEXT "$(TEXT_IO_TITLE)" "$(TEXT_IO_SUBTITLE)"
  again:
  !insertmacro MUI_INSTALLOPTIONS_DISPLAY_RETURN "init.ini"
  Pop ${TEMP}
  StrCmp $R0 "success" "" done
  ; test if something was seleted
  !insertmacro MUI_INSTALLOPTIONS_READ $R0 "init.ini" "Field 3" "State"
  !insertmacro MUI_INSTALLOPTIONS_READ $R1 "init.ini" "Field 4" "State"
  StrCmp $R0 "0" "" done
    StrCmp $R1 "0" "" done
      MessageBox MB_OK "Select what you want to do."
      Goto again
  done:
FunctionEnd

Function SetPage2
  !insertmacro MUI_INSTALLOPTIONS_READ ${TEMP} "init.ini" "Field 3" "State"
  StrCmp ${TEMP} "1" "" done
    ;generate config files
    call SetGenerateSetupPage
    return
  done:

  !insertmacro MUI_INSTALLOPTIONS_READ ${TEMP} "init.ini" "Field 4" "State"
  StrCmp ${TEMP} "1" "" done2
    ; install server or service
    call SetInstallPage
    return
  done2:

FunctionEnd

Function SetPage3
  !insertmacro MUI_INSTALLOPTIONS_READ ${TEMP} "init.ini" "Field 3" "State"
  StrCmp ${TEMP} "1" "" done
    ;generate page3
    call SetEnterIpAddresses
  done:
FunctionEnd

Function SetPage4
  !insertmacro MUI_INSTALLOPTIONS_READ ${TEMP} "init.ini" "Field 3" "State"
  StrCmp ${TEMP} "1" "" done
    ;generate page4
    call SetEnterPassword
  done:
FunctionEnd

Function SetPage5
  !insertmacro MUI_INSTALLOPTIONS_READ ${TEMP} "init.ini" "Field 3" "State"
  StrCmp ${TEMP} "1" "" done
    ;generate page5
    call SetGenerateProcessPage
  done:
FunctionEnd
;--------------------------------
; generate pages

Function SetGenerateSetupPage
  !insertmacro MUI_INSTALLOPTIONS_WRITE "generate_setup.ini" "Field 6" "State" "$INSTDIR\generated"
  !insertmacro MUI_HEADER_TEXT "$(TEXT_IO_TITLE)" "$(TEXT_IO_SUBTITLE)"

  again:
  !insertmacro MUI_INSTALLOPTIONS_DISPLAY_RETURN "generate_setup.ini"
  Pop ${TEMP}
  StrCmp $R0 "success" "" done
  ; test if something was seleted
  !insertmacro MUI_INSTALLOPTIONS_READ $R0 "generate_setup.ini" "Field 4" "State"
  IntCmp $R0 "1" "" error ""
  IntCmp $R0 "8" "" "" error
  goto done
  error:
    MessageBox MB_OK "Number of servers - enter value 1 - 8"
    Goto again
  done:

FunctionEnd

Function SetEnterPassword
  !insertmacro MUI_HEADER_TEXT "$(TEXT_IO_TITLE)" "Enter administrator password"
  again:
  !insertmacro MUI_INSTALLOPTIONS_DISPLAY_RETURN "password.ini"
  Pop $R0
  StrCmp $R0 "success" "" done
  ; test if password are the same
  !insertmacro MUI_INSTALLOPTIONS_READ $R0 "password.ini" "Field 3" "State"
  !insertmacro MUI_INSTALLOPTIONS_READ $R1 "password.ini" "Field 5" "State"
  StrCmp $R0 $R1 done ""
    MessageBox MB_OK "Passwords differs. Enter password again."
    Goto again
  done:
FunctionEnd


Function SetGenerateProcessPage
  ; set global varible $0 - generated directory
  !insertmacro MUI_INSTALLOPTIONS_READ $0 "generate_setup.ini" "Field 6" "State"
  ClearErrors
  CreateDirectory $0
  IFErrors "" next2
    MessageBox MB_OK "Cannot create directory $0 . Installation aborted."
    Quit
  next2:
  SetOutPath "$0"

  ClearErrors
  File "generate\gencfg.exe"
  Call copy_msvc_dll_files
  IfErrors "" done2
  MessageBox MB_OK "Error: Cannot copy temporary files to $0 . Installation aborted."
  Quit
  done2:

  !insertmacro MUI_INSTALLOPTIONS_READ $R0 "generate_setup.ini" "Field 4" "State"
  !insertmacro MUI_INSTALLOPTIONS_READ $R2 "password.ini" "Field 3" "State"
  StrCmp $R2 "" next ""
    StrCpy $R2 ' -p "$R2"' ;add password
  next:
  
  SetOutPath "$0"
  call GenerateGenCfgParameter
  Pop $R1
  ;ExecWait '"cmd.exe" /K echo "-s $R0 -n -i "$R1"$R2"' ${TEMP}
  ;Abort
  ExecWait '"$0\gencfg.exe" -s $R0 -n -i "$R1"$R2' ${TEMP}
  StrCmp ${TEMP} 0 "" error
    !insertmacro MUI_INSTALLOPTIONS_WRITE "generate_process.ini" "Field 1" "Text" "Simulation config files successfully generated."
    !insertmacro MUI_INSTALLOPTIONS_WRITE "generate_process.ini" "Field 2" "Text" "Copy content of generated directory to other machines to install server or service."
    Goto done
  error:
  !insertmacro MUI_INSTALLOPTIONS_WRITE "generate_process.ini" "Field 1" "Text" "Error occured while generating simulation config files."
  !insertmacro MUI_INSTALLOPTIONS_WRITE "generate_process.ini" "Field 2" "Text" ""
  MessageBox MB_OK "Error: Cannot generate config files. Installation aborted."
  Quit

  done:
  Delete "$0\gencfg.exe"
  Push "$0"
  Call delete_msvc_dll_files_used_for_gencfg

  call load_nsis_file

  !insertmacro MUI_HEADER_TEXT "$(TEXT_IO_TITLE)" "$(TEXT_IO_SUBTITLE)"
  !insertmacro MUI_INSTALLOPTIONS_DISPLAY "generate_process.ini"
  !insertmacro MUI_INSTALLOPTIONS_WRITE "init.ini" "Field 3" "State" "0"
FunctionEnd

Function GenerateGenCfgParameter
  Push $R0
  Push $R1
  Push $R2
  Push $R3
  Push $R5

  !insertmacro MUI_INSTALLOPTIONS_READ $R0 "generate_setup.ini" "Field 4" "State"
  ; number of servers is now in $R1

  ;add service
  !insertmacro MUI_INSTALLOPTIONS_READ $R2 "setup.ini" "Field 5" "State"
  !insertmacro MUI_INSTALLOPTIONS_READ $R3 "setup.ini" "Field 6" "State"
  Push "$R2 $R3"
  Pop $R8

  ; add servers
  Push "1"
  Pop $R1

  loop:
  IntCmp $R0 $R1 "" done ""

    ;count field numbers
    IntOp $R5 $R1 * "3"
    IntOp $R5 $R5 + "5"

    !insertmacro MUI_INSTALLOPTIONS_READ $R2 "setup.ini" "Field $R5" "State"
    IntOp $R5 $R5 + "1"
    !insertmacro MUI_INSTALLOPTIONS_READ $R3 "setup.ini" "Field $R5" "State"
    Push "$R8 $R2 $R3"
    Pop $R8

    IntOp $R1 $R1 + "1"
    goto loop
  done:
  Pop $R5
  Pop $R3
  Pop $R2
  Pop $R1
  Pop $R0
  ; save result $R8
  Push $R8
FunctionEnd


Function SetEnterIpAddresses
  call SetVisibleAllServers
  !insertmacro MUI_INSTALLOPTIONS_READ $R1 "generate_setup.ini" "Field 4" "State"
  IntOp $R1 $R1 + "1"
  Push $R1
  call SetInvisibleServersFrom
  !insertmacro MUI_HEADER_TEXT "$(TEXT_IO_TITLE)" "Enter ip address and ports for nodes"
  !insertmacro MUI_INSTALLOPTIONS_DISPLAY "setup.ini"
FunctionEnd

Function SetVisibleAllServers
  Push "1"
  Pop $R1
  loop:
  IntCmp 8 $R1 "" n1 ""
    push $R1
    call SetVisibleServer
    IntOp $R1 $R1 + "1"
    goto loop
  n1:
FunctionEnd

Function SetInvisibleServersFrom
  Pop $R1
  loop:
  IntCmp 8 $R1 "" n1 ""
    push $R1
    call SetInvisibleServer
    IntOp $R1 $R1 + "1"
    goto loop
  n1:
FunctionEnd

Function SetVisibleServer
  Pop ${TEMP} ; number of visible server nodes
  IntOp ${TEMP} ${TEMP} * "3"
  IntOp ${TEMP} ${TEMP} + "4"
  ; now I have number of the first field
  !insertmacro MUI_INSTALLOPTIONS_WRITE "setup.ini" "Field ${TEMP}" "Flags" ""
  IntOp ${TEMP} ${TEMP} + "1"
  !insertmacro MUI_INSTALLOPTIONS_WRITE "setup.ini" "Field ${TEMP}" "Flags" ""
  IntOp ${TEMP} ${TEMP} + "1"
  !insertmacro MUI_INSTALLOPTIONS_WRITE "setup.ini" "Field ${TEMP}" "Flags" ""
FunctionEnd

Function SetInvisibleServer
  Pop ${TEMP} ; number of visible server nodes
  IntOp ${TEMP} ${TEMP} * "3"
  IntOp ${TEMP} ${TEMP} + "4"
  ; now I have number of the first field
  !insertmacro MUI_INSTALLOPTIONS_WRITE "setup.ini" "Field ${TEMP}" "Flags" "DISABLED"
  IntOp ${TEMP} ${TEMP} + "1"
  !insertmacro MUI_INSTALLOPTIONS_WRITE "setup.ini" "Field ${TEMP}" "Flags" "DISABLED"
  IntOp ${TEMP} ${TEMP} + "1"
  !insertmacro MUI_INSTALLOPTIONS_WRITE "setup.ini" "Field ${TEMP}" "Flags" "DISABLED"
FunctionEnd

;--------------------------------
; install pages

!macro select_components service_count server_count
  Push ${service_count}
  pop $R0 ; number of service nodes

  Push ${server_count}
  pop $R1 ; number of server nodes

  ; show sections first
  SectionSetText ${SecServerUI1} "Server 1"
  SectionSetText ${SecServerUI2} "Server 2"
  SectionSetText ${SecServerUI3} "Server 3"
  SectionSetText ${SecServerUI4} "Server 4"
  SectionSetText ${SecServerUI5} "Server 5"
  SectionSetText ${SecServerUI6} "Server 6"
  SectionSetText ${SecServerUI7} "Server 7"
  SectionSetText ${SecServerUI8} "Server 8"

  ; hide sections
  IntCmp 1 $R1 n1 n1 ""
    SectionSetText ${SecServerUI1} ""
  n1:
  IntCmp 2 $R1 n2 n2 ""
    SectionSetText ${SecServerUI2} ""
  n2:
  IntCmp 3 $R1 n3 n3 ""
    SectionSetText ${SecServerUI3} ""
  n3:
  IntCmp 4 $R1 n4 n4 ""
    SectionSetText ${SecServerUI4} ""
  n4:
  IntCmp 5 $R1 n5 n5 ""
    SectionSetText ${SecServerUI5} ""
  n5:
  IntCmp 6 $R1 n6 n6 ""
    SectionSetText ${SecServerUI6} ""
  n6:
  IntCmp 7 $R1 n7 n7 ""
    SectionSetText ${SecServerUI7} ""
  n7:
  IntCmp 8 $R1 n8 n8 ""
    SectionSetText ${SecServerUI8} ""
  n8:

  ; set everything as not selected.
  SectionSetFlags ${SecServiceUI} SECTION_OFF

  SectionSetFlags ${SecServerUI1} SECTION_OFF
  SectionSetFlags ${SecServerUI2} SECTION_OFF
  SectionSetFlags ${SecServerUI3} SECTION_OFF
  SectionSetFlags ${SecServerUI4} SECTION_OFF
  SectionSetFlags ${SecServerUI5} SECTION_OFF
  SectionSetFlags ${SecServerUI6} SECTION_OFF
  SectionSetFlags ${SecServerUI7} SECTION_OFF
  SectionSetFlags ${SecServerUI8} SECTION_OFF
!macroend

Function SetInstallPage
  !insertmacro MUI_INSTALLOPTIONS_WRITE "install.ini" "Field 2" "State" "$INSTDIR\generated"
  !insertmacro MUI_HEADER_TEXT "Install service or server" "$(TEXT_IO_SUBTITLE)"
  again:
  !insertmacro MUI_INSTALLOPTIONS_DISPLAY_RETURN "install.ini"
  pop ${TEMP}
  StrCmp ${TEMP} "back" "" next
    ; go back
    return
  next:
  StrCmp ${TEMP} "success" "" next2
    ;check if generated config files exist
    !insertmacro MUI_INSTALLOPTIONS_READ $0 "install.ini" "Field 2" "State"
    ClearErrors
    FileOpen $R0 "$0\nsis" "r"
    IfErrors error
    ;FileOpen $R0 "$R1\config_nodes_public" "r"
    ;IfErrors error
    FileClose $R0
    Goto done
  error:
    MessageBox MB_OK "Error: selected directory doesn't contain all requested config files."
    goto again
  done:
  ; config files are ok.
  call load_nsis_file

  next2:
FunctionEnd

Function load_nsis_file
  ClearErrors
  FileOpen $R0 "$0\nsis" "r"
  IfErrors error
  FileRead $R0 $R1
  IfErrors error
  FileClose $R0
  !insertmacro select_components 1 $R1
  Goto done
  error:
    MessageBox MB_OK "Cannot read file nsis."
  done:
FunctionEnd

;Function SetInstall2Page
;  !insertmacro MUI_INSTALLOPTIONS_WRITE "install2.ini" "Field 2" "State" "$INSTDIR\generated"
;  !insertmacro MUI_HEADER_TEXT "" "$(TEXT_IO_SUBTITLE)"
;  !insertmacro MUI_INSTALLOPTIONS_DISPLAY "install2.ini"
;FunctionEnd

Function SetSetupServerPage
;   StrCpy $0 "server"
;  !insertmacro MUI_INSTALLOPTIONS_WRITE "install.ini" "Field 2" "State" "$INSTDIR\generated"
;  !insertmacro MUI_HEADER_TEXT "Setup server" "$(TEXT_IO_SUBTITLE)"
;  !insertmacro MUI_INSTALLOPTIONS_DISPLAY "install.ini"
FunctionEnd

Function SetSetupServicePage
;   StrCpy $0 "service"
;  !insertmacro MUI_HEADER_TEXT "Setup service" "$(TEXT_IO_SUBTITLE)"
;  !insertmacro MUI_INSTALLOPTIONS_DISPLAY "install.ini"
FunctionEnd

;--------------------------------
;Uninstaller Section

Section "Uninstall"

  ; delete startmenu
  SetShellVarContext all

  ReadRegStr $0 "${MUI_STARTMENUPAGE_REGISTRY_ROOT}" "${MUI_STARTMENUPAGE_REGISTRY_KEY}" "${MUI_STARTMENUPAGE_REGISTRY_VALUENAME}"
  StrCmp $0 "" noshortcuts
    Delete "$SMPROGRAMS\$0\Master data service.lnk"
    Delete "$SMPROGRAMS\$0\Server 1.lnk"
    Delete "$SMPROGRAMS\$0\Server 2.lnk"
    Delete "$SMPROGRAMS\$0\Server 3.lnk"
    Delete "$SMPROGRAMS\$0\Server 4.lnk"
    Delete "$SMPROGRAMS\$0\Server 5.lnk"
    Delete "$SMPROGRAMS\$0\Server 6.lnk"
    Delete "$SMPROGRAMS\$0\Server 7.lnk"
    Delete "$SMPROGRAMS\$0\Server 8.lnk"
    Delete "$SMPROGRAMS\$0\Start all.lnk"
    Delete "$SMPROGRAMS\$0\Uninstall.lnk"
    RMDir "$SMPROGRAMS\$0"
  noshortcuts:

  ReadRegDWORD $R1 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "Service1"
  StrCmp $R1 "1" "" next
    ; delete Service1
    Delete "$INSTDIR\master_data_service\login\default"
    RMDir "$INSTDIR\master_data_service\login"

    Push "$INSTDIR\master_data_service"
    Call un.delete_msvc_dll_files
    
    Delete "$INSTDIR\master_data_service\massiv.conf"
    Delete "$INSTDIR\master_data_service\massiv.paths.conf"
    Delete "$INSTDIR\master_data_service\massiv.settings.conf"
    Delete "$INSTDIR\master_data_service\demo_service.exe"
    RMDIr /r "$INSTDIR\master_data_service\source_data"
    RMDIr /r "$INSTDIR\master_data_service\data"
    RMDIr /r "$INSTDIR\master_data_service"
  next:
  Push "1"
  Pop $R0
  loop:
    IntCmp $R0 8 "" "" finito
    ReadRegDWORD $R1 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "Server$R0"
    StrCmp $R1 "1" "" not_installed
      ; delete Server$R0
      Delete "$INSTDIR\server$R0\login\default"
      RMDir "$INSTDIR\server$R0\login"
      
      Push "$INSTDIR\server$R0"
      Call un.delete_msvc_dll_files

      Delete "$INSTDIR\server$R0\massiv.conf"
      Delete "$INSTDIR\server$R0\massiv.paths.conf"
      Delete "$INSTDIR\server$R0\massiv.settings.conf"
      Delete "$INSTDIR\server$R0\demo_server.exe"
      Delete "$INSTDIR\server$R0\swap_file" ; hack
      RMDIr /r "$INSTDIR\server$R0\data"
      RMDIr /r "$INSTDIR\server$R0\archives"
      RMDIr /r "$INSTDIR\server$R0"
    not_installed:
    IntOp $R0 $R0 + "1"
  Goto loop
  finito:

  Delete "$INSTDIR\start_all.bat"
  Delete "$INSTDIR\massiv.logger.conf"
  Delete "$INSTDIR\massiv.replication.conf"
  Delete "$INSTDIR\massiv.server_nodes.conf"
  Delete "$INSTDIR\Uninstall.exe"
  RMDir "$INSTDIR"

  ; delete registry
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}"

  !insertmacro MUI_UNFINISHHEADER

SectionEnd

;--------------------------------
;Callback functions

Function .onInit
  ; test if user have right to install/uninstall programs on this machine
  ClearErrors
  UserInfo::GetName
  IfErrors Win9x
  Pop $R0
  ClearErrors
  UserInfo::GetAccountType
  IfErrors Win9x
  Pop $R1
  StrCmp $R1 "" done "" ; win98 (tested)
  StrCmp $R1 "Admin" done ""
  StrCmp $R1 "Power" done ""
    MessageBox MB_OK "You dont have rights to install software on this machine."
    abort
  Win9x: ; no problemo with admin on win9x
  done:

  ReadRegStr $R0 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "DisplayName"
  StrCmp $R0 "" noerror
    MessageBox MB_OK "${MUI_PRODUCT} already installed. Uninstall it first."
    Abort
  noerror:

  ;Extract InstallOptions INI Files
  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "init.ini"
  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "generate_setup.ini"
  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "generate_process.ini"
  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "install.ini"
  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "setup.ini"
  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "password.ini"
FunctionEnd


Function un.onInit
  call un.test_registry
FunctionEnd

;--------------------------------
;User functions

Function un.test_registry
  ;Read registry values
  ReadRegStr $R0 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "DisplayName"
  StrCmp $R0 "" "" noerror
    MessageBox MB_OK "Error during uninstallation. Registry values are corrupted."
    Abort
  noerror:
FunctionEnd

